package lightcontainer.domains.client;

import lightcontainer.storage.AppData;
import lightcontainer.storage.File;
import lightcontainer.storage.Repository;
import lightcontainer.storage.User;
import lightcontainer.utils.AES_GCM;
import lightcontainer.utils.ShaHasher;

import java.security.NoSuchAlgorithmException;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * Contexte associé à la requête d'un utilisateur.
 * Permet de récupérer les données à celui-ci et d'effectuer des actions sur le context courant.
 */
public class Context {

    private Repository repository;

    private RequestBundle requestBundle;

    /**
     * Login de l'utilisateur
     */
    private String login;

    /**
     * Domain s'occupant de la requête
     */
    private String domain;

    // Constructeur
    public Context(Repository repository) {
        this.repository = repository;
    }


    /**
     * Permet de créer un utilisateur.
     *
     * @param login    Login de l'utilisateur
     * @param password Mot de passe de l'utilisateur
     * @return TRUE si l'utilisateur a pu être créé
     */
    public boolean createUser(String login, String password) {
        try {
            String key = AES_GCM.generateSecretKey();

            ShaHasher hasher = new ShaHasher(password);
            password = hasher.nextHashing();
            String passwordSalt = hasher.getSalt();

            if (this.repository.addUser(login, password, key, passwordSalt)) {
                this.login = login;
                return true;
            }
        } catch (AES_GCM.AesGcmException e) {
        }
        return false;
    }

    /**
     * Login de l'utilisateur
     *
     * @return Login de l'utilisateur
     */
    public String getLogin() {
        return this.login;
    }


    /**
     * Permet de demander la connection de l'utilisateur
     *
     * @param login    Login
     * @param password Mot de passe
     * @return TRUE si l'utilisateur a été authentifié
     */
    public boolean signIn(String login, String password) {
        String passwordSalt = this.repository.getUserPasswordSalt(login);
        if (passwordSalt != null) {
            ShaHasher hasher = new ShaHasher(password);

            if (this.repository.verifyUser(login, hasher.fromSalt(hasher.saltToByte(passwordSalt)))) {
                this.login = login;
                return true;
            }
        }
        return false;
    }

    /**
     * Permet de savoir si l'utilisateur s'est connecté (Avec login et mdp)
     *
     * @return TRUE si l'utilisateur est connecté
     */
    public boolean isConnected() {
        return this.getLogin() != null;
    }


    /**
     * Clé AES de l'utilisateur
     *
     * @return Clé AES de l'utilisateur
     */
    public String getAesKey() {
        return this.repository.getUserAesKey(getLogin());
    }

    /**
     * Permet de régénérer le Requestbundle courant. Méthode appelée lorsque l'on souhaite
     */
    public void newBundle() {
        requestBundle = new RequestBundle();
    }

    /**
     * Permet d'ajouter des données pour la requête courrante
     *
     * @param key   La clé permettant de retrouver la valeur
     * @param value La valeur associée à la clé
     */
    public void putDataString(String key, String value) {
        requestBundle.putString(key, value);
    }

    /**
     * Permet d'ajouter des données pour la requête courrante
     *
     * @param key   La clé permettant de retrouver la valeur
     * @param value La valeur associée à la clé
     */
    public void putDataInt(String key, int value) {
        requestBundle.putInt(key, value);
    }

    /**
     * Permet de récupérer des données pour la requête courrante
     *
     * @param key La clé permettant de retrouver la valeur
     * @return La valeur associée à la clé ou null
     */
    public String getDataString(String key) {
        return requestBundle.getString(key);
    }

    /**
     * Permet de récupérer des données pour la requête courrante
     *
     * @param key La clé permettant de retrouver la valeur
     * @return La valeur associée à la clé
     */
    public int getDataInt(String key) {
        return requestBundle.getInt(key);
    }

    /**
     * Permet d'ajouter un fichier à l'utilisateur
     *
     * @param fileName     Nom du fichier hashé
     * @param fileNameSalt Salt appliqué sur le nom du fichier
     * @param size         Taille du fichier
     * @param iv           IV du fichier
     * @param domain       Domain dans lequel est stocké le fichier
     * @return TRUE si le fichier a pu être enregistré.
     */
    public boolean addFile(String fileName, String fileNameSalt, int size, String iv, String domain) {
        return this.repository.addFileFor(new File(fileName, fileNameSalt, size, iv, Set.of(domain)), getLogin());
    }

    public List<String> getStringifiedFilesOf() {
        return repository.load().getStringifiedFilesOf(login);
    }

    public boolean canAddFile() {
        return repository.canAddFile(login);
    }

    public String getDomain() {
        return this.domain;
    }

    public void setDomain(String domain) {
        this.domain = domain;
    }
}
